@@ -206,8 +206,13 @@ getBits (int nBits, Int64 &c, int &lc, const char *&in)
 //	- see http://www.compressconsult.com/huffman/
 //
 
+#if !defined (OPENEXR_IMF_HAVE_LARGE_STACK)
+void
+hufCanonicalCodeTable (Int64 *hcode)
+#else
 void
 hufCanonicalCodeTable (Int64 hcode[HUF_ENCSIZE])
+#endif
 {
     Int64 n[59];
 
@@ -263,11 +268,19 @@ hufCanonicalCodeTable (Int64 hcode[HUF_ENCSIZE])
 //	- original frequencies are destroyed;
 //	- encoding tables are used by hufEncode() and hufBuildDecTable();
 //
+// NB: The following code "(*a == *b) && (a > b))" was added to ensure
+//     elements in the heap with the same value are sorted by index.
+//     This is to ensure, the STL make_heap()/pop_heap()/push_heap() methods
+//     produced a resultant sorted heap that is identical across OSes.
+//
 
 
 struct FHeapCompare
 {
-    bool operator () (Int64 *a, Int64 *b) {return *a > *b;}
+    bool operator () (Int64 *a, Int64 *b)
+    {
+        return ((*a > *b) || ((*a == *b) && (a > b)));
+    }
 };
 
 
@@ -897,6 +910,11 @@ hufDecode
 		//
 
 		lc -= pl.len;
+
+		if ( lc < 0 )
+		{
+			invalidCode(); // code length too long
+		}
 		getCode (pl.lit, rlc, c, lc, in, out, outb, oe);
 	    }
 	    else
@@ -954,6 +972,10 @@ hufDecode
 	if (pl.len)
 	{
 	    lc -= pl.len;
+            if ( lc < 0 )
+            {
+   	        invalidCode(); // code length too long
+            }
 	    getCode (pl.lit, rlc, c, lc, in, out, outb, oe);
 	}
 	else
@@ -967,10 +989,17 @@ hufDecode
 }
 
 
+#if !defined (OPENEXR_IMF_HAVE_LARGE_STACK)
+void
+countFrequencies (Int64 *freq,
+		  const unsigned short data[/*n*/],
+		  int n)
+#else
 void
 countFrequencies (Int64 freq[HUF_ENCSIZE],
 		  const unsigned short data[/*n*/],
 		  int n)
+#endif
 {
     for (int i = 0; i < HUF_ENCSIZE; ++i)
 	freq[i] = 0;
@@ -1073,7 +1102,9 @@ hufUncompress (const char compressed[],
 
     const char *ptr = compressed + 20;
 
-    if ( ptr + (nBits+7 )/8 > compressed+nCompressed)
+    uint64_t nBytes = (static_cast<uint64_t>(nBits)+7) / 8 ;
+
+    if ( ptr + nBytes > compressed+nCompressed)
     {
         notEnoughData();
         return;
