@@ -149,6 +149,11 @@ struct TileBuffer
      TileBuffer (Compressor * const comp);
     ~TileBuffer ();
 
+    TileBuffer (const TileBuffer& other) = delete;
+    TileBuffer& operator = (const TileBuffer& other) = delete;
+    TileBuffer (TileBuffer&& other) = delete;
+    TileBuffer& operator = (TileBuffer&& other) = delete;
+
     inline void		wait () {_sem.wait();}
     inline void		post () {_sem.post();}
 
@@ -243,6 +248,11 @@ struct TiledInputFile::Data: public Mutex
      Data (int numThreads);
     ~Data ();
 
+    Data (const Data& other) = delete;
+    Data& operator = (const Data& other) = delete;
+    Data (Data&& other) = delete;
+    Data& operator = (Data&& other) = delete;
+
     inline TileBuffer * getTileBuffer (int number);
 					    // hash function from tile indices
 					    // into our vector of tile buffers
@@ -255,6 +265,7 @@ TiledInputFile::Data::Data (int numThreads):
     partNumber (-1),
     multiPartBackwardSupport(false),
     numThreads(numThreads),
+    multiPartFile(nullptr),
     memoryMapped(false),
     _streamData(NULL),
     _deleteStream(false)
@@ -377,7 +388,7 @@ readTileData (InputStreamMutex *streamData,
     if (levelY != ly)
         throw IEX_NAMESPACE::InputExc ("Unexpected tile y level number coordinate.");
 
-    if (dataSize > (int) ifd->tileBufferSize)
+    if (dataSize < 0 || dataSize > static_cast<int>(ifd->tileBufferSize) )
         throw IEX_NAMESPACE::InputExc ("Unexpected tile block length.");
 
     //
@@ -744,7 +755,7 @@ TiledInputFile::TiledInputFile (const char fileName[], int numThreads):
                 }
             }
         }
-        if ( _data->_streamData != 0)
+        if ( _data->_streamData != 0 && !isMultiPart(_data->version))
         {
             delete _data->_streamData->is;
             _data->_streamData->is = is = 0;
@@ -807,7 +818,10 @@ TiledInputFile::TiledInputFile (OPENEXR_IMF_INTERNAL_NAMESPACE::IStream &is, int
         {
             for (size_t i = 0; i < _data->tileBuffers.size(); i++)
             {
-                delete [] _data->tileBuffers[i]->buffer;
+                if( _data->tileBuffers[i])
+                {
+                   delete [] _data->tileBuffers[i]->buffer;
+                }
             }
         }
         if (streamDataCreated) delete _data->_streamData;
@@ -849,12 +863,15 @@ TiledInputFile::TiledInputFile (const Header &header,
         {
             for (size_t i = 0; i < _data->tileBuffers.size(); i++)
             {
-                delete [] _data->tileBuffers[i]->buffer;
+                if(_data->tileBuffers[i])
+                {
+                     delete [] _data->tileBuffers[i]->buffer;
+                }
             }
         }
         delete _data->_streamData;
-	delete _data;
-        throw;
+        delete _data;
+        throw; 
     }
 }
 
@@ -869,7 +886,20 @@ TiledInputFile::TiledInputFile (InputPartData* part)
     }
     catch(...)
     {
-        if (_data) delete _data;
+        if(_data)
+        {
+          if (!_data->memoryMapped)
+          {
+            for (size_t i = 0; i < _data->tileBuffers.size(); i++)
+            {
+                if(_data->tileBuffers[i])
+                {
+                   delete [] _data->tileBuffers[i]->buffer;
+                }
+            }
+          }
+          delete _data;
+        }
         throw;
     }
 }
@@ -928,7 +958,10 @@ TiledInputFile::initialize ()
     {
         if (!isTiled (_data->version))
             throw IEX_NAMESPACE::ArgExc ("Expected a tiled file but the file is not tiled.");
-        
+
+        if (isNonImage (_data->version))
+            throw IEX_NAMESPACE::ArgExc ("File is not a regular tiled image.");
+
     }
     else
     {
@@ -969,6 +1002,16 @@ TiledInputFile::initialize ()
 
     _data->tileBufferSize = _data->maxBytesPerTileLine * _data->tileDesc.ySize;
 
+    //
+    // OpenEXR has a limit of INT_MAX compressed bytes per tile
+    // disallow uncompressed tile sizes above INT_MAX too to guarantee file is written
+    //
+    if( _data->tileBufferSize > INT_MAX )
+    {
+        throw IEX_NAMESPACE::ArgExc ("Tile size too large for OpenEXR format");
+    }
+
+
     //
     // Create all the TileBuffers and allocate their internal buffers
     //
@@ -1274,7 +1317,7 @@ TiledInputFile::readTiles (int dx1, int dx2, int dy1, int dy2, int lx, int ly)
     catch (IEX_NAMESPACE::BaseExc &e)
     {
         REPLACE_EXC (e, "Error reading pixel data from image "
-                        "file \"" << fileName() << "\". " << e);
+                     "file \"" << fileName() << "\". " << e.what());
         throw;
     }
 }
@@ -1332,6 +1375,11 @@ TiledInputFile::rawTileData (int &dx, int &dy,
         readNextTileData (_data->_streamData, _data, dx, dy, lx, ly,
 			  tileBuffer->buffer,
                           pixelDataSize);
+
+        if ( !isValidLevel(lx,ly) || !isValidTile (dx, dy, lx, ly) )
+            throw IEX_NAMESPACE::ArgExc ("File contains an invalid tile");
+
+
         if(isMultiPart(version()))
         {
             if (old_dx!=dx || old_dy !=dy || old_lx!=lx || old_ly!=ly)
@@ -1339,12 +1387,19 @@ TiledInputFile::rawTileData (int &dx, int &dy,
                 throw IEX_NAMESPACE::ArgExc ("rawTileData read the wrong tile");
             }
         }
+        else
+        {
+             if(!isValidTile (dx, dy, lx, ly) )
+             {
+                 throw IEX_NAMESPACE::IoExc ("rawTileData read an invalid tile");
+             }
+        }
         pixelData = tileBuffer->buffer;
     }
     catch (IEX_NAMESPACE::BaseExc &e)
     {
         REPLACE_EXC (e, "Error reading pixel data from image "
-			"file \"" << fileName() << "\". " << e);
+                     "file \"" << fileName() << "\". " << e.what());
         throw;
     }
 }
@@ -1432,7 +1487,7 @@ TiledInputFile::levelWidth (int lx) const
     catch (IEX_NAMESPACE::BaseExc &e)
     {
 	REPLACE_EXC (e, "Error calling levelWidth() on image "
-			"file \"" << fileName() << "\". " << e);
+                 "file \"" << fileName() << "\". " << e.what());
 	throw;
     }
 }
@@ -1449,7 +1504,7 @@ TiledInputFile::levelHeight (int ly) const
     catch (IEX_NAMESPACE::BaseExc &e)
     {
 	REPLACE_EXC (e, "Error calling levelHeight() on image "
-			"file \"" << fileName() << "\". " << e);
+                 "file \"" << fileName() << "\". " << e.what());
 	throw;
     }
 }
@@ -1505,7 +1560,7 @@ TiledInputFile::dataWindowForLevel (int lx, int ly) const
     catch (IEX_NAMESPACE::BaseExc &e)
     {
 	REPLACE_EXC (e, "Error calling dataWindowForLevel() on image "
-			"file \"" << fileName() << "\". " << e);
+                 "file \"" << fileName() << "\". " << e.what());
 	throw;
     }
 }
@@ -1535,7 +1590,7 @@ TiledInputFile::dataWindowForTile (int dx, int dy, int lx, int ly) const
     catch (IEX_NAMESPACE::BaseExc &e)
     {
 	REPLACE_EXC (e, "Error calling dataWindowForTile() on image "
-			"file \"" << fileName() << "\". " << e);
+                 "file \"" << fileName() << "\". " << e.what());
 	throw;
     }
 }
