@@ -38,16 +38,18 @@
 // order to work around MSVC limitations.
 //
 
-#include <PyImathColor.h>
-#include <PyImathVec.h>
+#include "PyIlmBaseConfigInternal.h"
+
+#include "PyImathColor.h"
+#include "PyImathVec.h"
 #include "PyImathDecorators.h"
 #include "PyImathExport.h"
 #include <Python.h>
 #include <boost/python.hpp>
 #include <boost/python/make_constructor.hpp>
 #include <boost/format.hpp>
-#include <PyImath.h>
-#include <PyImathMathExc.h>
+#include "PyImath.h"
+#include "PyImathMathExc.h"
 #include <ImathVec.h>
 #include <ImathColor.h>
 #include <ImathColorAlgo.h>
@@ -593,8 +595,8 @@ register_Color3()
         .def_readwrite("b", &Color3<T>::z)
         .def("__str__", &color3_str<T>)
         .def("__repr__", &color3_repr<T>)
-        .def(self == self)
-        .def(self != self)
+        .def(self == self) // NOSONAR - suppress SonarCloud bug report.
+        .def(self != self) // NOSONAR - suppress SonarCloud bug report.
         .def("__iadd__", &iadd<T>,return_internal_reference<>())
         .def("__add__", &add<T>)
         .def("__add__", &addTuple<T>)
