@@ -32,9 +32,11 @@
 //
 ///////////////////////////////////////////////////////////////////////////
 
+#include "PyIlmBaseConfigInternal.h"
+
 #define BOOST_PYTHON_MAX_ARITY 17
 
-#include <PyImathMatrix.h>
+#include "PyImathMatrix.h"
 #include "PyImathExport.h"
 #include "PyImathDecorators.h"
 #include <Python.h>
@@ -44,13 +46,13 @@
 #include <boost/python/tuple.hpp>
 #include <boost/python/dict.hpp>
 #include <boost/python/raw_function.hpp>
-#include <PyImath.h>
-#include <PyImathVec.h>
-#include <PyImathMathExc.h>
+#include "PyImath.h"
+#include "PyImathVec.h"
+#include "PyImathMathExc.h"
 #include <ImathVec.h>
 #include <ImathMatrixAlgo.h>
 #include <Iex.h>
-#include <PyImathTask.h>
+#include "PyImathTask.h"
 
 namespace PyImath {
 template<> const char PYIMATH_EXPORT *PyImath::M44fArray::name() { return "M44fArray"; }
@@ -1001,11 +1003,11 @@ register_Matrix44()
         .def("fastMinor",&Matrix44<T>::fastMinor,"fastMinor() return matrix minor using the specified rows and columns of this matrix")
         .def("determinant",&Matrix44<T>::determinant,"determinant() return the determinant of this matrix")
         .def("invert",&invert44<T>,invert44_overloads("invert() invert this matrix")[return_internal_reference<>()])
-        .def("inverse",&inverse44<T>,inverse44_overloads("inverse() return a inverted copy of this matrix"))
+        .def("inverse",&inverse44<T>,inverse44_overloads("inverse() return an inverted copy of this matrix"))
         .def("gjInvert",&gjInvert44<T>,gjInvert44_overloads("gjInvert() invert this matrix")[return_internal_reference<>()])
-        .def("gjInverse",&gjInverse44<T>,gjInverse44_overloads("gjInverse() return a inverted copy of this matrix"))
-        .def(self == self)
-        .def(self != self)
+        .def("gjInverse",&gjInverse44<T>,gjInverse44_overloads("gjInverse() return an inverted copy of this matrix"))
+        .def(self == self) // NOSONAR - suppress SonarCloud bug report.
+        .def(self != self) // NOSONAR - suppress SonarCloud bug report.
         .def("__iadd__", &iadd44<T, float>,return_internal_reference<>())
         .def("__iadd__", &iadd44<T, double>,return_internal_reference<>())
         .def("__iadd__", &iadd44T<T>,return_internal_reference<>())
